@@ -7,6 +7,7 @@ from torch.nn.functional import linear
 import torch.jit
 
 from ..modules import Linear, LayerNorm, Embedding, TransformerEncoder, TransformerDecoder
+from ..utils import transposed_cat
 
 
 class Blender(torch.nn.Module):
@@ -31,6 +32,7 @@ class Blender(torch.nn.Module):
         )
 
         self.classifier_head = torch.nn.Linear(dim, num_labels) if num_labels else None
+        self.scale = math.sqrt(dim)
 
         self.n_decoder_layers = n_decoder_layers
         self.n_heads = n_heads
@@ -59,7 +61,7 @@ class Blender(torch.nn.Module):
         return model.half()
 
     def embedding_forward(self, tensor: torch.Tensor, positions: torch.Tensor) -> torch.Tensor:
-        return self.embeddings(tensor) + self.position_embeddings(positions)
+        return self.embeddings(tensor) * self.scale + self.position_embeddings(positions)
 
     @torch.jit.export
     def encode(self, encoder_ids: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor]:
@@ -81,6 +83,7 @@ class Blender(torch.nn.Module):
 
         positions = mask.cumsum(dim=1, dtype=torch.long)
         if incr_state is not None:
+            incr_state = incr_state.flatten(0, 1)
             decoder_ids = decoder_ids[:, -self.decode_step:]
             positions = positions[:, -self.decode_step:]
 
@@ -108,25 +111,20 @@ class Blender(torch.nn.Module):
             n_heads=n_heads, n_layers=n_encoder_layers, embedding_size=dim, ffn_size=ffn_size, blender_norm=True
         )
 
-    def convert_old_state_dict(self, state_dict: dict):
+    @classmethod
+    def convert_old_state_dict(cls, state_dict: dict, n_encoder_layers: int, n_decoder_layers: int):
         for p in 'weight', 'bias':
             for module in 'encoder', 'decoder':
                 state_dict[f'{module}_norm.{p}'] = state_dict.pop(f'{module}.norm_embeddings.{p}')
             state_dict[f'pre_decoder_layer.{p}'] = torch.cat(
                 [state_dict.pop(f'decoder.layers.{ind}.encoder_attention.{name}_lin.{p}')
-                 for ind in range(len(self.decoder.layers)) for name in 'kv'], dim=0)
-
-            scale = math.sqrt(self.encoder.layers[0].attention.dim_per_head)
-
-            for ind in range(len(self.encoder.layers)):
-                state_dict[f'encoder.layers.{ind}.attention.q_lin.{p}'] /= scale
-                state_dict[f'encoder.layers.{ind}.attention.pre_attention.{p}'] = torch.cat(
-                    [state_dict.pop(f'encoder.layers.{ind}.attention.{name}_lin.{p}') for name in 'qkv'], dim=0)
-            for ind in range(len(self.decoder.layers)):
-                state_dict[f'decoder.layers.{ind}.self_attention.q_lin.{p}'] /= scale
-                state_dict[f'decoder.layers.{ind}.self_attention.pre_attention.{p}'] = torch.cat(
-                    [state_dict.pop(f'decoder.layers.{ind}.self_attention.{name}_lin.{p}') for name in 'qkv'], dim=0)
-                state_dict[f'decoder.layers.{ind}.encoder_attention.q_lin.{p}'] /= scale
+                 for ind in range(n_decoder_layers) for name in 'kv'], dim=0)
+            for ind in range(n_encoder_layers):
+                state_dict[f'encoder.layers.{ind}.attention.pre_attention.{p}'] = transposed_cat(
+                    [state_dict.pop(f'encoder.layers.{ind}.attention.{name}_lin.{p}') for name in 'qkv'])
+            for ind in range(n_decoder_layers):
+                state_dict[f'decoder.layers.{ind}.self_attention.pre_attention.{p}'] = transposed_cat(
+                    [state_dict.pop(f'decoder.layers.{ind}.self_attention.{name}_lin.{p}') for name in 'qkv'])
 
         state_dict['position_embeddings.weight'] = state_dict['encoder.position_embeddings.weight']
 
@@ -148,9 +146,8 @@ class Blender(torch.nn.Module):
                 state_dict.pop(f'classifier_head.{p}', None)
 
         if 'encoder.pre_decoder_layer.weight' not in state_dict:
-            self.convert_old_state_dict(state_dict)
-
-        state_dict['embeddings.weight'] *= math.sqrt(self.embeddings.embedding_dim)
+            self.convert_old_state_dict(
+                state_dict, n_encoder_layers=len(self.encoder.layers), n_decoder_layers=len(self.decoder.layers))
 
         super().load_state_dict(state_dict, strict)
 
