@@ -66,12 +66,15 @@ class SessionState:
 
 
 class ModelServer:
-    def __init__(self, port: int, model: torch.nn.Module, model_name: str):
-        logger.info('Server starting')
+    __slots__ = ('model', 'device', 'server_socket', 'verbose')
+
+    def __init__(self, port: int, model: torch.nn.Module, verbose: bool):
+        logger.info(f'{type(model).__name__} model server starting on port {port}')
         self.model = model
         self.device = next(self.model.parameters()).device
         self.server_socket = self.create_server_socket(port)
-        logger.info(f'{model_name} model server started on port {port}')
+        self.verbose = verbose
+        logger.info(f'{type(model).__name__} model server started on port {port}')
 
     @staticmethod
     def create_server_socket(port: int) -> socket.socket:
@@ -172,14 +175,18 @@ class ModelServer:
             try:
                 keys = list(inputs[0].keys())
                 sizes = [sample_inputs[keys[0]].shape[0] for sample_inputs in inputs]
-                print(f'INPUT LENGTHS: {sizes}')
                 inputs = {
                     key: self.pad_and_concat(list(map(itemgetter(key), inputs)), pad_map.get(key, 0))
                     for key in keys
                 }
-                print(f'CONCATTED SHAPE: {inputs[keys[0]].shape}')
+                start_time = time.time() if self.verbose else None
                 with torch.no_grad():
                     outputs = self.model.forward(**inputs)
+                if self.verbose:
+                    logger.debug(
+                        f'Processed inputs of sizes: {sizes}\n'
+                        f'Total size: {sum(sizes)}'
+                        f'\nTime spent: {time.time() - start_time}')
                 for session_state, output in zip(session_states, self.split_output(outputs, sizes)):
                     session_state.send_output(output)
             except Exception as e:
